/**
 * @file led_pwm.c
 * @author
 * @brief
 * @version 0.1
 * @date 2023-06-09
 *
 * @copyright Copyright (c) 2023
 *
 */

/* Includes --------------------------------------------------------------------------------------*/
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <nrfx_pwm.h>
#include <hal/nrf_gpio.h>
#include "app_common.h"
#include "led_pwm.h"

/* Private param ---------------------------------------------------------------------------------*/
static nrfx_pwm_t LED_PWM_INST = NRFX_PWM_INSTANCE(1);

/* Beep tone used to beep when search is requested */
static nrf_pwm_values_individual_t led_on_values[] = {
/* chn_0  chn_1  chnl_2  counter top */
  {0,       0,      0,      250},
};

/* Beep tone used to beep when search is requested */
static nrf_pwm_values_individual_t led_fast_blink_values[] = {
/* chn_0  chn_1  chnl_2  counter top */
  {0,       0,      0,      250},
  {0,       0,      50,     250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      50,     250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
};

static nrf_pwm_values_individual_t led_zone_change_blink_values[] = {
/* chn_0  chn_1  chnl_2  counter top */
  {0,       0,      0,      250},
  {0,       0,      25,     250},
  {0,       0,      50,     250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      50,     250},
  {0,       0,      25,     250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
};

/* Beep tone used to beep when search is requested */
static nrf_pwm_values_individual_t led_slow_blink_values[] = {
/* chn_0  chn_1  chnl_2  counter top */
  {0,       0,      0,      250},
  {0,       0,      25,     250},
  {0,       0,      50,     250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      100,    250},
  {0,       0,      50,     250},
  {0,       0,      25,     250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
  {0,       0,      0,      250},
};

/* Global param ----------------------------------------------------------------------------------*/
K_TIMER_DEFINE(led_timer, NULL, NULL);

nrfx_pwm_config_t const config1 = {
    .output_pins =
    {
    	LED_R_PIO,					// channel 0
    	LED_G_PIO,					// channel 1
    	LED_B_PIO,					// channel 2
	    NRF_PWM_PIN_NOT_CONNECTED,	// channel 3
    },

	.pin_inverted =
	{
		true,
		true,
		true,
		false,
	},

    .irq_priority = 7,
    .base_clock   = NRF_PWM_CLK_1MHz,
    .count_mode   = NRF_PWM_MODE_UP,
    .top_value    = 250,
    .load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
    .step_mode    = NRF_PWM_STEP_AUTO
};

nrf_pwm_sequence_t led_seq =
{
    .values.p_individual = led_on_values,
    .length              = NRF_PWM_VALUES_LENGTH(led_on_values),
    .repeats             = 100,
    .end_delay           = 0
};

/**
 * @brief LED timer timeout callback function
 *
 * @param p_timer
 */
void cb_led_timer (struct k_timer *p_timer)
{
	led_off();
}

/**
 * @brief Initialize LED
 *
 */
void led_init (void)
{
	uint8_t err = 0;
	uint32_t out_pins[NRF_PWM_CHANNEL_COUNT] = {
        LED_R_PIO, LED_G_PIO, LED_B_PIO, NRF_PWM_PIN_NOT_CONNECTED
    };

    nrf_pwm_pins_set(LED_PWM_INST.p_reg, out_pins);
	err = nrfx_pwm_init(&LED_PWM_INST, &config1, NULL, NULL);
    k_timer_init(&led_timer, cb_led_timer, NULL);
}

/**
 * @brief Turn OFF LED PWM
 *
 */
void led_off (void)
{
	nrfx_pwm_stop(&LED_PWM_INST, true);
	while (!nrfx_pwm_stopped_check(&LED_PWM_INST));
}

/**
 * @brief Set LED color as requested using PWM
 *
 * @param r - red PWM value
 * @param g - green PWM value
 * @param b - blue PWM value
 */
void led_set_color (uint16_t r, uint16_t g, uint16_t b)
{
	led_off();

	led_on_values[0].channel_0  = r;
	led_on_values[0].channel_1  = g;
	led_on_values[0].channel_2  = b;
	led_seq.values.p_individual = led_on_values,
	led_seq.length              = NRF_PWM_VALUES_LENGTH(led_on_values),

	nrfx_pwm_simple_playback(&LED_PWM_INST, &led_seq, 2, NRFX_PWM_FLAG_LOOP);
}

/**
 * @brief Blink requested LED color fast using PWM
 *
 * @param r - red PWM value
 * @param g - green PWM value
 * @param b - blue PWM value
 */
void led_fast_blink (uint16_t r, uint16_t g, uint16_t b)
{
	led_off();

    nrf_pwm_values_individual_t *p = led_fast_blink_values;
	led_seq.values.p_individual = led_fast_blink_values;
	led_seq.length              = NRF_PWM_VALUES_LENGTH(led_fast_blink_values);
	led_seq.repeats             = 100;

	p[0].channel_0 =  0;    p[0].channel_1 = 0;   p[0].channel_2 = 0;
	p[1].channel_0 =  r/2;  p[1].channel_1 = g/2; p[1].channel_2 = b/2;
	p[2].channel_0 =  r;    p[2].channel_1 = g;   p[2].channel_2 = b;
	p[3].channel_0 =  r;    p[3].channel_1 = g;   p[3].channel_2 = b;
	p[4].channel_0 =  r;    p[4].channel_1 = g;   p[4].channel_2 = b;
	p[5].channel_0 =  r;    p[5].channel_1 = g;   p[5].channel_2 = b;
	p[6].channel_0 =  r/2;  p[6].channel_1 = g/2; p[6].channel_2 = b/2;
	p[7].channel_0 =  0;    p[7].channel_1 = 0;   p[7].channel_2 = 0;
	p[8].channel_0 =  0;    p[8].channel_1 = 0;   p[8].channel_2 = 0;
	p[9].channel_0 =  0;    p[9].channel_1 = 0;   p[9].channel_2 = 0;

	nrfx_pwm_simple_playback(&LED_PWM_INST, &led_seq, 2, NRFX_PWM_FLAG_LOOP);
}
/**
 * @brief Blink requested LED color slow using PWM for zone or mode change
 *
 * @param r - red PWM value
 * @param g - green PWM value
 * @param b - blue PWM value
 */
void led_zone_or_mode_change_sequence (uint16_t r, uint16_t g, uint16_t b)
{
	led_off();

	nrf_pwm_values_individual_t *p = led_zone_change_blink_values;

	led_seq.values.p_individual = led_zone_change_blink_values,
	led_seq.length              = NRF_PWM_VALUES_LENGTH(led_zone_change_blink_values),
	led_seq.repeats             = 300;

	p[0].channel_0 =  0;    p[0].channel_1 = 0;   p[0].channel_2 = 0;
	p[1].channel_0 =  r/4;  p[1].channel_1 = g/4; p[1].channel_2 = b/4;
	p[2].channel_0 =  r/2;  p[2].channel_1 = g/2; p[2].channel_2 = b/2;
	p[3].channel_0 =  r;    p[3].channel_1 = g;   p[3].channel_2 = b;
	p[4].channel_0 =  r;    p[4].channel_1 = g;   p[4].channel_2 = b;
	p[5].channel_0 =  r;    p[5].channel_1 = g;   p[5].channel_2 = b;
	p[6].channel_0 =  r;    p[6].channel_1 = g;   p[6].channel_2 = b;
	p[7].channel_0 =  r;    p[7].channel_1 = g;   p[7].channel_2 = b;
	p[8].channel_0 =  r/2;  p[8].channel_1 = g/2; p[8].channel_2 = b/2;
	p[9].channel_0 =  r/4;  p[9].channel_1 = g/4; p[9].channel_2 = b/4;
	p[10].channel_0 =  0;   p[10].channel_1 = 0;  p[10].channel_2 = 0;
	p[11].channel_0 =  0;   p[11].channel_1 = 0;  p[11].channel_2 = 0;
	p[12].channel_0 =  0;   p[12].channel_1 = 0;  p[12].channel_2 = 0;
	p[13].channel_0 =  0;   p[13].channel_1 = 0;  p[13].channel_2 = 0;
	p[14].channel_0 =  0;   p[14].channel_1 = 0;  p[14].channel_2 = 0;

	nrfx_pwm_simple_playback(&LED_PWM_INST, &led_seq, 2, NRFX_PWM_FLAG_LOOP);
}
/**
 * @brief Blink requested LED color slow using PWM
 *
 * @param r - red PWM value
 * @param g - green PWM value
 * @param b - blue PWM value
 */
void led_slow_blink (uint16_t r, uint16_t g, uint16_t b)
{
	led_off();

	nrf_pwm_values_individual_t *p = led_slow_blink_values;

	led_seq.values.p_individual = led_slow_blink_values,
	led_seq.length              = NRF_PWM_VALUES_LENGTH(led_slow_blink_values),
	led_seq.repeats             = 300;

	p[0].channel_0 =  0;    p[0].channel_1 = 0;   p[0].channel_2 = 0;
	p[1].channel_0 =  r/4;  p[1].channel_1 = g/4; p[1].channel_2 = b/4;
	p[2].channel_0 =  r/2;  p[2].channel_1 = g/2; p[2].channel_2 = b/2;
	p[3].channel_0 =  r;    p[3].channel_1 = g;   p[3].channel_2 = b;
	p[4].channel_0 =  r;    p[4].channel_1 = g;   p[4].channel_2 = b;
	p[5].channel_0 =  r;    p[5].channel_1 = g;   p[5].channel_2 = b;
	p[6].channel_0 =  r;    p[6].channel_1 = g;   p[6].channel_2 = b;
	p[7].channel_0 =  r;    p[7].channel_1 = g;   p[7].channel_2 = b;
	p[8].channel_0 =  r/2;  p[8].channel_1 = g/2; p[8].channel_2 = b/2;
	p[9].channel_0 =  r/4;  p[9].channel_1 = g/4; p[9].channel_2 = b/4;
	p[10].channel_0 =  0;   p[10].channel_1 = 0;  p[10].channel_2 = 0;
	p[11].channel_0 =  0;   p[11].channel_1 = 0;  p[11].channel_2 = 0;
	p[12].channel_0 =  0;   p[12].channel_1 = 0;  p[12].channel_2 = 0;
	p[13].channel_0 =  0;   p[13].channel_1 = 0;  p[13].channel_2 = 0;
	p[14].channel_0 =  0;   p[14].channel_1 = 0;  p[14].channel_2 = 0;

	nrfx_pwm_simple_playback(&LED_PWM_INST, &led_seq, 2, NRFX_PWM_FLAG_LOOP);
}

/**
 * @brief Indicate the device set zone range using LED
 *
 */
void turn_on_zone_indicator_led (void)
{
	if (dev_info.zone_range == ZONE_1) {
		led_zone_or_mode_change_sequence(LED_COLOR_ZONE_1);
		k_timer_start(&led_timer, K_SECONDS(LED_TIMEOUT_SEC), K_NO_WAIT);
	}
  else if (dev_info.zone_range == ZONE_2) {
		led_zone_or_mode_change_sequence(LED_COLOR_ZONE_2);
		k_timer_start(&led_timer, K_SECONDS(LED_TIMEOUT_SEC), K_NO_WAIT);
	}
  else if (dev_info.zone_range == ZONE_3) {
		led_zone_or_mode_change_sequence(LED_COLOR_ZONE_3);
		k_timer_start(&led_timer, K_SECONDS(LED_TIMEOUT_SEC), K_NO_WAIT);
	}
  else if (dev_info.zone_range == ZONE_SILENT) {
		led_zone_or_mode_change_sequence(LED_COLOR_ZONE_SILENT);
		k_timer_start(&led_timer, K_SECONDS(LED_TIMEOUT_SEC), K_NO_WAIT);
	}
}

